@@ -1,6 +1,7 @@
 #!/usr/bin/env bash
 
-#set -eu
+set -eu
+set -o pipefail
 
 : '
 
@@ -10,67 +11,78 @@ todo
 - shrink icu data
 '
 
+MASON_VERSION="v0.5.0"
+
 function setup_mason() {
     if [[ ! -d ./.mason ]]; then
-        git clone --depth 1 https://github.com/mapbox/mason.git ./.mason
+        git clone https://github.com/mapbox/mason.git ./.mason
+        (cd ./.mason && git checkout ${MASON_VERSION})
     else
         echo "Updating to latest mason"
-        (cd ./.mason && git pull)
+        (cd ./.mason && git fetch && git checkout ${MASON_VERSION} && git pull origin ${MASON_VERSION})
     fi
-    export MASON_DIR=$(pwd)/.mason
     export PATH=$(pwd)/.mason:$PATH
     export CXX=${CXX:-clang++}
     export CC=${CC:-clang}
 }
 
-if [[ $(uname -s) == 'Darwin' ]]; then
-    FIND_PATTERN="\/Users\/travis\/build\/mapbox\/mason"
-else
-    FIND_PATTERN="\/home\/travis\/build\/mapbox\/mason"
-fi
-
-REPLACE="$(pwd)"
-REPLACE=${REPLACE////\\/}
-
 function install() {
     MASON_PLATFORM_ID=$(mason env MASON_PLATFORM_ID)
     if [[ ! -d ./mason_packages/${MASON_PLATFORM_ID}/${1}/${2} ]]; then
         mason install $1 $2
-        mason link $1 $2
-        if [[ $3 ]]; then
-            LA_FILE=$(${MASON_DIR:-~/.mason}/mason prefix $1 $2)/lib/$3.la
+        if [[ ${3:-false} != false ]]; then
+            LA_FILE=$(mason prefix $1 $2)/lib/$3.la
             if [[ -f ${LA_FILE} ]]; then
-               perl -i -p -e "s/${FIND_PATTERN}/${REPLACE}/g;" ${LA_FILE}
+                perl -i -p -e 's:\Q$ENV{HOME}/build/mapbox/mason\E:$ENV{PWD}:g' ${LA_FILE}
             else
                 echo "$LA_FILE not found"
             fi
         fi
     fi
+    # the rm here is to workaround https://github.com/mapbox/mason/issues/230
+    rm -f ./mason_packages/.link/mason.ini
+    mason link $1 $2
 }
 
 ICU_VERSION="55.1"
 
 function install_mason_deps() {
-    install gdal 1.11.2 libgdal &
-    install boost 1.59.0 &
-    install boost_liball 1.59.0 &
-    install freetype 2.6 libfreetype &
-    install harfbuzz 0.9.40 libharfbuzz &
-    install jpeg_turbo 1.4.0 libjpeg &
-    install libpng 1.6.17 libpng &
-    install webp 0.4.2 libwebp &
+    FAIL=0
+    install ccache 3.3.0 &
+    install zlib system &
+    install jpeg_turbo 1.5.0 libjpeg &
+    install libpng 1.6.24 libpng &
+    install libtiff 4.0.6 libtiff &
+    install libpq 9.5.2 &
+    install sqlite 3.14.1 libsqlite3 &
+    install expat 2.2.0 libexpat &
     install icu ${ICU_VERSION} &
-    install proj 4.8.0 libproj &
-    install libtiff 4.0.4beta libtiff &
-    install libpq 9.4.0 &
-    install sqlite 3.8.8.1 libsqlite3 &
-    install expat 2.1.0 libexpat &
-    install pixman 0.32.6 libpixman-1 &
-    install cairo 1.14.2 libcairo &
+    install proj 4.9.2 libproj &
+    install pixman 0.34.0 libpixman-1 &
+    install cairo 1.14.6 libcairo &
     install protobuf 2.6.1 &
-    wait
     # technically protobuf is not a mapnik core dep, but installing
     # here by default helps make mapnik-vector-tile builds easier
+    install webp 0.5.1 libwebp &
+    install gdal 2.1.1 libgdal &
+    install boost 1.61.0 &
+    install boost_libsystem 1.61.0 &
+    install boost_libfilesystem 1.61.0 &
+    install boost_libprogram_options 1.61.0 &
+    install boost_libregex_icu 1.61.0 &
+    # technically boost thread and python are not a core dep, but installing
+    # here by default helps make python-mapnik builds easier
+    install boost_libthread 1.61.0 &
+    install boost_libpython 1.61.0 &
+    install freetype 2.6.5 libfreetype &
+    install harfbuzz 1.3.0 libharfbuzz &
+    for job in $(jobs -p)
+    do
+        wait $job || let "FAIL+=1"
+    done
+    if [[ "$FAIL" != "0" ]]; then
+        exit ${FAIL}
+    fi
 }
 
 MASON_LINKED_ABS=$(pwd)/mason_packages/.link
@@ -80,22 +92,16 @@ export CPLUS_INCLUDE_PATH="${MASON_LINKED_ABS}/include"
 export LIBRARY_PATH="${MASON_LINKED_ABS}/lib"
 
 function make_config() {
-    if [[ $(uname -s) == 'Darwin' ]]; then
-        local PATH_REPLACE="/Users/travis/build/mapbox/mason/mason_packages:./mason_packages"
-    else
-        local PATH_REPLACE="/home/travis/build/mapbox/mason/mason_packages:./mason_packages"
-    fi
-
     echo "
 CXX = '$CXX'
 CC = '$CC'
-CUSTOM_CXXFLAGS = '-fvisibility=hidden -fvisibility-inlines-hidden -DU_CHARSET_IS_UTF8=1'
+CUSTOM_CXXFLAGS = '-D_GLIBCXX_USE_CXX11_ABI=0'
 RUNTIME_LINK = 'static'
 INPUT_PLUGINS = 'all'
 PATH = '${MASON_LINKED_REL}/bin'
 PKG_CONFIG_PATH = '${MASON_LINKED_REL}/lib/pkgconfig'
 PATH_REMOVE = '/usr:/usr/local'
-PATH_REPLACE = '${PATH_REPLACE}'
+PATH_REPLACE = '$HOME/build/mapbox/mason/mason_packages:./mason_packages'
 BOOST_INCLUDES = '${MASON_LINKED_REL}/include'
 BOOST_LIBS = '${MASON_LINKED_REL}/lib'
 ICU_INCLUDES = '${MASON_LINKED_REL}/include'
@@ -126,8 +132,7 @@ CPP_TESTS = True
 PGSQL2SQLITE = True
 XMLPARSER = 'ptree'
 SVG2PNG = True
-SAMPLE_INPUT_PLUGINS = True
-" > ./config.py
+"
 }
 
 # NOTE: the `mapnik-settings.env` is used by test/run (which is run by `make test`)
@@ -141,7 +146,7 @@ function setup_runtime_settings() {
 function main() {
     setup_mason
     install_mason_deps
-    make_config
+    make_config > ./config.py
     setup_runtime_settings
     echo "Ready, now run:"
     echo ""
@@ -149,3 +154,8 @@ function main() {
 }
 
 main
+
+# allow sourcing of script without
+# causing the terminal to bail on error
+set +eu
+set +o pipefail
