@@ -21,7 +21,9 @@
  *****************************************************************************/
 
 #include "catch.hpp"
+#include "ds_test_util.hpp"
 
+#include <mapnik/unicode.hpp>
 #include <mapnik/datasource.hpp>
 #include <mapnik/datasource_cache.hpp>
 #include <mapnik/geometry.hpp>
@@ -61,25 +63,6 @@ std::pair<mapnik::datasource_ptr,mapnik::feature_ptr> fetch_first_feature(std::s
     return std::make_pair(ds,feature);
 }
 
-int create_disk_index(std::string const& filename, bool silent = true)
-{
-    std::string cmd;
-    if (std::getenv("DYLD_LIBRARY_PATH") != nullptr)
-    {
-        cmd += std::string("export DYLD_LIBRARY_PATH=") + std::getenv("DYLD_LIBRARY_PATH") + " && ";
-    }
-    cmd += "mapnik-index " + filename;
-    if (silent)
-    {
-#ifndef _WINDOWS
-        cmd += " 2>/dev/null";
-#else
-        cmd += " 2> nul";
-#endif
-    }
-    return std::system(cmd.c_str());
-}
-
 }
 
 TEST_CASE("geojson") {
@@ -113,6 +96,97 @@ TEST_CASE("geojson") {
             }
         }
 
+        SECTION("GeoJSON an empty FeatureCollection")
+        {
+            for (auto cache_features : {true, false})
+            {
+                mapnik::parameters params;
+                params["type"] = "geojson";
+                params["file"] = "./test/data/json/empty_featurecollection.json";
+                params["cache_features"] = cache_features;
+                auto ds = mapnik::datasource_cache::instance().create(params);
+                CHECK(ds != nullptr);
+                auto fs = all_features(ds);
+                REQUIRE(!mapnik::is_valid(fs));
+                while (auto f = fs->next())
+                {
+                    CHECK(false); // shouldn't get here
+                }
+            }
+        }
+
+        SECTION("GeoJSON num_features_to_query")
+        {
+            std::string filename = "./test/data/json/featurecollection-multipleprops.geojson";
+            for (mapnik::value_integer num_features_to_query : { mapnik::value_integer(-1),
+                        mapnik::value_integer(0),
+                        mapnik::value_integer(1),
+                        mapnik::value_integer(2),
+                        mapnik::value_integer(3),
+                        std::numeric_limits<mapnik::value_integer>::max()})
+            {
+                for (auto create_index : { true, false })
+                {
+                    if (create_index)
+                    {
+                        int ret = create_disk_index(filename);
+                        int ret_posix = (ret >> 8) & 0x000000ff;
+                        INFO(ret);
+                        INFO(ret_posix);
+                        CHECK(mapnik::util::exists(filename + ".index"));
+                    }
+
+                    for (auto cache_features : {true, false})
+                    {
+                        mapnik::parameters params;
+                        params["type"] = "geojson";
+                        params["file"] = filename;
+                        params["cache_features"] = cache_features;
+                        params["num_features_to_query"] = num_features_to_query;
+                        auto ds = mapnik::datasource_cache::instance().create(params);
+                        CHECK(ds != nullptr);
+                        auto fields = ds->get_descriptor().get_descriptors();
+                        if (!create_index && cache_features)
+                        {
+                            // when there's no index and caching is enabled descriptor is always fully initialised
+                            REQUIRE(fields.size() == 2);
+                        }
+                        else
+                        {
+                            // at least 1 feature should be queried
+                            REQUIRE(fields.size() == std::min(std::max(mapnik::value_integer(1), num_features_to_query),
+                                                              mapnik::value_integer(2)));
+                        }
+                    }
+                    // cleanup
+                    if (create_index && mapnik::util::exists(filename + ".index"))
+                    {
+                        mapnik::util::remove(filename + ".index");
+                    }
+                }
+            }
+        }
+
+        SECTION("GeoJSON attribute descriptors are alphabetically ordered")
+        {
+            for (auto cache_features : {true, false})
+            {
+                mapnik::parameters params;
+                params["type"] = "geojson";
+                params["file"] = "./test/data/json/properties.json";
+                params["cache_features"] = cache_features;
+                auto ds = mapnik::datasource_cache::instance().create(params);
+                CHECK(ds != nullptr);
+                std::vector<std::string> expected_names = {"a", "b", "c", "d", "e"};
+                auto fields = ds->get_descriptor().get_descriptors();
+                std::size_t index = 0;
+                for (auto const& field : fields)
+                {
+                    REQUIRE(field.get_name() == expected_names[index++]);
+                }
+            }
+        }
+
         SECTION("GeoJSON invalid Point")
         {
             for (auto cache_features : {true, false})
@@ -302,8 +376,6 @@ TEST_CASE("geojson") {
                     }
                     auto features = ds->features(query);
                     auto features2 = ds->features_at_point(ds->envelope().center(),0);
-                    REQUIRE(features != nullptr);
-                    REQUIRE(features2 != nullptr);
                     auto feature = features->next();
                     auto feature2 = features2->next();
                     REQUIRE(feature != nullptr);
@@ -421,7 +493,6 @@ TEST_CASE("geojson") {
                         query.add_property_name(field.get_name());
                     }
                     auto features = ds->features(query);
-                    REQUIRE(features != nullptr);
                     auto feature = features->next();
                     REQUIRE(feature != nullptr);
                     REQUIRE(feature->envelope() == mapnik::box2d<double>(123,456,123,456));
@@ -443,7 +514,8 @@ TEST_CASE("geojson") {
 
             for (auto const& c_str : {"./test/data/json/feature-malformed-1.geojson",
                         "./test/data/json/feature-malformed-2.geojson",
-                        "./test/data/json/feature-malformed-3.geojson"})
+                        "./test/data/json/feature-malformed-3.geojson",
+                        "./test/data/json/feature-malformed-4.geojson"})
             {
                 std::string filename(c_str);
                 params["file"] = filename;
@@ -483,43 +555,46 @@ TEST_CASE("geojson") {
 
         SECTION("GeoJSON ensure mapnik::featureset::next() throws on malformed input")
         {
-            std::string filename{"./test/data/json/featurecollection-malformed.json"};
             mapnik::parameters params;
             params["type"] = "geojson";
-            params["file"] = filename;
-
-            // cleanup in the case of a failed previous run
-            if (mapnik::util::exists(filename + ".index"))
+            for (auto const& c_str : {"./test/data/json/featurecollection-malformed.json",
+                        "./test/data/json/featurecollection-malformed-2.json"})
             {
-                mapnik::util::remove(filename + ".index");
-            }
+                std::string filename(c_str);
+                params["file"] = filename;
+                // cleanup in the case of a failed previous run
+                if (mapnik::util::exists(filename + ".index"))
+                {
+                    mapnik::util::remove(filename + ".index");
+                }
 
-            CHECK(!mapnik::util::exists(filename + ".index"));
-            int ret = create_disk_index(filename);
-            int ret_posix = (ret >> 8) & 0x000000ff;
-            INFO(ret);
-            INFO(ret_posix);
-            CHECK(mapnik::util::exists(filename + ".index"));
+                CHECK(!mapnik::util::exists(filename + ".index"));
+                int ret = create_disk_index(filename);
+                int ret_posix = (ret >> 8) & 0x000000ff;
+                INFO(ret);
+                INFO(ret_posix);
+                CHECK(mapnik::util::exists(filename + ".index"));
 
-            for (auto cache_features : {true,false})
-            {
-                params["cache_features"] = cache_features;
-                auto ds = mapnik::datasource_cache::instance().create(params);
-                auto fields = ds->get_descriptor().get_descriptors();
-                mapnik::query query(ds->envelope());
-                auto features = ds->features(query);
-                REQUIRE_THROWS(
-                    auto feature = features->next();
-                    while (feature != nullptr)
-                    {
-                        feature = features->next();
-                    });
-            }
+                for (auto cache_features : {true,false})
+                {
+                    params["cache_features"] = cache_features;
+                    auto ds = mapnik::datasource_cache::instance().create(params);
+                    auto fields = ds->get_descriptor().get_descriptors();
+                    mapnik::query query(ds->envelope());
+                    auto features = ds->features(query);
+                    REQUIRE_THROWS(
+                        auto feature = features->next();
+                        while (feature != nullptr)
+                        {
+                            feature = features->next();
+                        });
+                }
 
-            // cleanup
-            if (mapnik::util::exists(filename + ".index"))
-            {
-                mapnik::util::remove(filename + ".index");
+                // cleanup
+                if (mapnik::util::exists(filename + ".index"))
+                {
+                    mapnik::util::remove(filename + ".index");
+                }
             }
         }
 
@@ -629,5 +704,114 @@ TEST_CASE("geojson") {
                 }
             }
         }
+
+        SECTION("GeoJSON descriptor returns all field names")
+        {
+            mapnik::parameters params;
+            params["type"] = "geojson";
+
+            std::string filename("./test/data/json/featurecollection-multipleprops.geojson");
+            params["file"] = filename;
+
+            // cleanup in the case of a failed previous run
+            if (mapnik::util::exists(filename + ".index"))
+            {
+                mapnik::util::remove(filename + ".index");
+            }
+
+            for (auto create_index : { true, false })
+            {
+                if (create_index)
+                {
+                    CHECK(!mapnik::util::exists(filename + ".index"));
+                    int ret = create_disk_index(filename);
+                    int ret_posix = (ret >> 8) & 0x000000ff;
+                    INFO(ret);
+                    INFO(ret_posix);
+                    CHECK(mapnik::util::exists(filename + ".index"));
+                }
+
+                for (auto cache_features : {true, false})
+                {
+                    params["cache_features"] = cache_features;
+                    auto ds = mapnik::datasource_cache::instance().create(params);
+                    REQUIRE(bool(ds));
+                    auto fields = ds->get_descriptor().get_descriptors();
+                    std::initializer_list<std::string> names = {"one", "two"};
+                    REQUIRE_FIELD_NAMES(fields, names);
+                }
+                // cleanup
+                if (create_index && mapnik::util::exists(filename + ".index"))
+                {
+                    mapnik::util::remove(filename + ".index");
+                }
+            }
+        }
+
+        SECTION("GeoJSON properties are properly expressed")
+        {
+            mapnik::transcoder tr("utf8");
+            mapnik::parameters params;
+            params["type"] = "geojson";
+
+            std::string filename("./test/data/json/escaped.geojson");
+            params["file"] = filename;
+
+            // cleanup in the case of a failed previous run
+            if (mapnik::util::exists(filename + ".index"))
+            {
+                mapnik::util::remove(filename + ".index");
+            }
+
+            for (auto create_index : { true, false })
+            {
+                if (create_index)
+                {
+                    CHECK(!mapnik::util::exists(filename + ".index"));
+                    int ret = create_disk_index(filename);
+                    int ret_posix = (ret >> 8) & 0x000000ff;
+                    INFO(ret);
+                    INFO(ret_posix);
+                    CHECK(mapnik::util::exists(filename + ".index"));
+                }
+
+                for (auto cache_features : {true, false})
+                {
+                    params["cache_features"] = cache_features;
+                    auto ds = mapnik::datasource_cache::instance().create(params);
+                    REQUIRE(bool(ds));
+                    auto fields = ds->get_descriptor().get_descriptors();
+                    std::initializer_list<std::string> names = {"NOM_FR","array","boolean","description","double","empty_array", "empty_object","int","name","object","spaces"};
+                    REQUIRE_FIELD_NAMES(fields, names);
+
+                    auto fs = all_features(ds);
+                    std::initializer_list<attr> attrs = {
+                        attr{"name", tr.transcode("Test")},
+                        attr{"NOM_FR", tr.transcode("Québec")},
+                        attr{"boolean", mapnik::value_bool("true")},
+                        attr{"description", tr.transcode("Test: \u005C")},
+                        attr{"double", mapnik::value_double(1.1)},
+                        attr{"int", mapnik::value_integer(1)},
+                        attr{"object", tr.transcode("{\"name\":\"waka\",\"spaces\":\"value with spaces\",\"int\":1,\"double\":1.1,\"boolean\":false"
+                                                    ",\"NOM_FR\":\"Québec\",\"array\":[\"string\",\"value with spaces\",3,1.1,null,true"
+                                                    ",\"Québec\"],\"another_object\":{\"name\":\"nested object\"}}")},
+                        attr{"spaces", tr.transcode("this has spaces")},
+                        attr{"array", tr.transcode("[\"string\",\"value with spaces\",3,1.1,null,true,"
+                                                   "\"Québec\",{\"name\":\"object within an array\"},"
+                                                   "[\"array\",\"within\",\"an\",\"array\"]]")},
+                        attr{"empty_array", tr.transcode("[]")},
+                        attr{"empty_object", tr.transcode("{}")},
+                    };
+                    auto feature = fs->next();
+                    REQUIRE(bool(feature));
+                    REQUIRE_ATTRIBUTES(feature, attrs);
+                }
+                // cleanup
+                if (create_index && mapnik::util::exists(filename + ".index"))
+                {
+                    mapnik::util::remove(filename + ".index");
+                }
+            }
+        }
     }
 }
